@@ -1,6 +1,6 @@
 #include <stdio.h>
 #include <assert.h>
-#include <urcu/uatomic_arch.h>
+#include <urcu/uatomic.h>
 
 struct testvals {
 	unsigned char c;
@@ -33,12 +33,16 @@ do {						\
 	assert(uatomic_read(ptr) == 23);		\
 	uatomic_dec(ptr);			\
 	assert(uatomic_read(ptr) == 22);		\
-	v = uatomic_add_return(ptr, 100);	\
-	assert(v == 122);			\
+	v = uatomic_add_return(ptr, 74);	\
+	assert(v == 96);			\
+	assert(uatomic_read(ptr) == 96);	\
+	uatomic_or(ptr, 58);			\
 	assert(uatomic_read(ptr) == 122);	\
 	v = uatomic_sub_return(ptr, 1);		\
 	assert(v == 121);			\
 	assert(uatomic_read(ptr) == 121);	\
+	uatomic_and(ptr, 129);			\
+	assert(uatomic_read(ptr) == 1);		\
 } while (0)
 
 int main(int argc, char **argv)
