@@ -32,7 +32,8 @@
 #include <pthread.h>
 
 /*
- * See urcu-pointer.h and urcu-pointer-static.h for pointer publication headers.
+ * See urcu-pointer.h and urcu/static/urcu-pointer.h for pointer
+ * publication headers.
  */
 #include <urcu-pointer.h>
 
@@ -40,6 +41,8 @@
 extern "C" {
 #endif 
 
+#include <urcu/map/urcu-qsbr.h>
+
 /*
  * Important !
  *
@@ -50,7 +53,7 @@ extern "C" {
 
 #ifdef _LGPL_SOURCE
 
-#include <urcu-qsbr-static.h>
+#include <urcu/static/urcu-qsbr.h>
 
 /*
  * Mappings for static use of the userspace RCU library.
@@ -62,15 +65,15 @@ extern "C" {
  * rcu_read_unlock()
  *
  * Mark the beginning and end of a read-side critical section.
- * DON'T FORGET TO USE rcu_register_thread/rcu_unregister_thread() FOR EACH
- * THREAD WITH READ-SIDE CRITICAL SECTION.
+ * DON'T FORGET TO USE rcu_register_thread/rcu_unregister_thread()
+ * FOR EACH THREAD WITH READ-SIDE CRITICAL SECTION.
  */
-#define rcu_read_lock()		_rcu_read_lock()
-#define rcu_read_unlock()	_rcu_read_unlock()
+#define rcu_read_lock_qsbr		_rcu_read_lock
+#define rcu_read_unlock_qsbr		_rcu_read_unlock
 
-#define rcu_quiescent_state()	_rcu_quiescent_state()
-#define rcu_thread_offline()	_rcu_thread_offline()
-#define rcu_thread_online()	_rcu_thread_online()
+#define rcu_quiescent_state_qsbr	_rcu_quiescent_state
+#define rcu_thread_offline_qsbr		_rcu_thread_offline
+#define rcu_thread_online_qsbr		_rcu_thread_online
 
 #else /* !_LGPL_SOURCE */
 
@@ -93,7 +96,7 @@ static inline void rcu_read_lock(void)
 {
 }
 
-static inline void rcu_read_lock(void)
+static inline void rcu_read_unlock(void)
 {
 }
 
@@ -122,4 +125,7 @@ extern void rcu_unregister_thread(void);
 }
 #endif
 
+#include <urcu-call-rcu.h>
+#include <urcu-defer.h>
+
 #endif /* _URCU_QSBR_H */
