@@ -107,6 +107,21 @@ extern void rcu_unregister_thread(void);
  */
 extern void rcu_init(void);
 
+/*
+ * Q.S. reporting are no-ops for these URCU flavors.
+ */
+static inline void rcu_quiescent_state(void)
+{
+}
+
+static inline void rcu_thread_offline(void)
+{
+}
+
+static inline void rcu_thread_online(void)
+{
+}
+
 #ifdef __cplusplus 
 }
 #endif
